import logging
import os
import subprocess
from datetime import datetime

from google.cloud import storage
from google.oauth2 import service_account

_logger = logging.getLogger(__name__)


def get_storage(json_account_info):
    credentials = service_account.Credentials.from_service_account_info(
        json_account_info
    )
    return storage.Client(credentials=credentials)


def get_bucket(storage_client, name):
    return storage_client.bucket(name)


def upload_blob(
    bucket_name, source_file_name, destination_blob_name, json_account_info
):
    """Uploads a file to the bucket."""
    # The ID of your GCS bucket
    # bucket_name = "your-bucket-name"
    # The path to your file to upload
    # source_file_name = "local/path/to/file"
    # The ID of your GCS object
    # destination_blob_name = "storage-object-name"

    _logger.error(json_account_info)

    storage_client = get_storage(json_account_info)
    bucket = get_bucket(storage_client, bucket_name)

    # storage_client = storage.Client()
    bucket = storage_client.bucket(bucket_name)
    blob = bucket.blob(destination_blob_name)

    # Optional: set a generation-match precondition to avoid potential race conditions
    # and data corruptions. The request to upload is aborted if the object's
    # generation number does not match your precondition. For a destination
    # object that does not yet exist, set the if_generation_match precondition to 0.
    # If the destination object already exists in your bucket, set instead a
    # generation-match precondition using its generation number.
    generation_match_precondition = 0

    blob.upload_from_filename(
        source_file_name, if_generation_match=generation_match_precondition
    )

    _logger.warning("File %s uploaded to %s.", source_file_name, destination_blob_name)


def backup_database(dbname, **kwargs):
    if not dbname:
        raise ValueError("Database name is mandatory.")

    filestore = kwargs.get("filestore", True)
    ttype = kwargs.get("ttype", "zip")
    prefix = kwargs.pop("prefix", False)
    path = kwargs.pop("path", "/tmp")
    args = ["click-odoo-backupdb"]

    if not filestore:
        args.append("--no-filestore")

    if ttype:
        args += ["--format", ttype]

    args.append(dbname)
    suffix = "obt_no_fs" if not filestore else "obt"

    filename = get_name(dbname, ttype, prefix, suffix)
    filepath = os.path.join(path, filename)

    if os.path.exists(filepath):
        os.remove(filepath)

    args.append(filepath)

    _logger.debug(args)

    try:
        subprocess.run(args)
        res = True
    except Exception as error:
        res = False
        _logger.error(error)

    return res, filename, filepath


def get_name(dbname, ttype, prefix=None, suffix=None):
    now = datetime.now()
    string_date = now.strftime("%Y-%m-%d_%H%M")

    if not suffix:
        suffix = "obt"

    res = f"{prefix or dbname}_{dbname}_{string_date}_{suffix}"

    if ttype == "zip":
        res += ".zip"
    elif ttype == "dump":
        res += ".dump"

    return res


def clean_files(filepath):
    if os.path.exists(filepath):
        os.remove(filepath)



import json
import os
from functools import lru_cache
import json
from pathlib import Path
from typing import Any, Dict, Tuple, Type, Optional

from pydantic.fields import FieldInfo

from pydantic_settings import (
    BaseSettings,
    PydanticBaseSettingsSource,
    SettingsConfigDict,
)

CONFIG_PATH = Path(".obt")
CONFIG_FILEPATH = Path(".obt", "config.json")


class JsonConfigSettingsSource(PydanticBaseSettingsSource):
    """
    A simple settings source class that loads variables from a JSON file
    at the project's root.

    Here we happen to choose to use the `env_file_encoding` from Config
    when reading `config.json
    """

    def get_field_value(
        self, field: FieldInfo, field_name: str
    ) -> Tuple[Any, str, bool]:
        encoding = self.config.get("env_file_encoding")
        file_content_json = json.loads(CONFIG_FILEPATH.read_text(encoding))
        field_value = file_content_json.get(field_name)
        return field_value, field_name, False

    def prepare_field_value(
        self, field_name: str, field: FieldInfo, value: Any, value_is_complex: bool
    ) -> Any:
        return value

    def __call__(self) -> Dict[str, Any]:
        d: Dict[str, Any] = {}
        encoding = self.config.get("env_file_encoding")

        if not CONFIG_PATH.exists():
            CONFIG_PATH.mkdir()

        if not CONFIG_FILEPATH.exists():
            with open(CONFIG_FILEPATH, "w", encoding=encoding) as f:
                default_values = {
                    key: os.getenv(key.upper()) or ""
                    for key in self.settings_cls.model_fields.keys()
                    if key != "auth"
                }
                f.write(json.dumps(default_values))

        for field_name, field in self.settings_cls.model_fields.items():
            field_value, field_key, value_is_complex = self.get_field_value(
                field, field_name
            )
            field_value = self.prepare_field_value(
                field_name, field, field_value, value_is_complex
            )
            if field_value is not None:
                d[field_key] = field_value

        return d


class Settings(BaseSettings):
    model_config = SettingsConfigDict(env_file_encoding="utf-8")
    version: str = "0.1.0"

    bucket_name: str
    env_json_auth: str
    default_database: str
    auth: Optional[dict] = {}

    @classmethod
    def settings_customise_sources(
        cls,
        settings_cls: Type[BaseSettings],
        init_settings: PydanticBaseSettingsSource,
        env_settings: PydanticBaseSettingsSource,
        dotenv_settings: PydanticBaseSettingsSource,
        file_secret_settings: PydanticBaseSettingsSource,
    ) -> Tuple[PydanticBaseSettingsSource, ...]:
        return (
            init_settings,
            JsonConfigSettingsSource(settings_cls),
            env_settings,
            file_secret_settings,
        )

    @property
    def json_auth(self) -> dict:
        if self.auth:
            return self.auth
        elif self.env_json_auth:
            json.loads(self.env_json_auth)
        return {}


# @lru_cache()
def get_settings():
    return Settings()



import sys

import click

from obt.core.settings import get_settings

settings = get_settings()  # pylint: disable=C0413

from obt.core.tools import backup_database, clean_files, upload_blob  # noqa: E402


@click.group()
def cli():
    """Odoo Backup Tool"""


@click.command()
@click.argument("dbname")
@click.option(
    "--format",
    "-f",
    required=True,
    type=str,
    help="Format: zip, dump, folder.",
)
@click.option(
    "--no-filestore",
    "-n",
    is_flag=True,
    default=False,
    type=bool,
    help="Do not include filestore.",
)
@click.option("--prefix", "-p", required=False, type=str, help="Use prefix")
def backup(dbname, no_filestore, **kwargs):
    options = {
        "filestore": not no_filestore,
        "ttype": kwargs.get("format"),
        "prefix": kwargs.get("prefix", False),
    }

    res, _, filepath = backup_database(dbname, **options)
    if not res:
        sys.exit(1)

    print(filepath)


@click.command()
@click.option(
    "--dbname",
    "-d",
    required=False,
    default=None,
    type=str,
    help="Database name.",
)
@click.option(
    "--format",
    "-f",
    required=True,
    type=str,
    help="Format: zip, dump, folder.",
)
@click.option(
    "--no-filestore",
    "-n",
    is_flag=True,
    default=False,
    type=bool,
    help="Do not include filestore.",
)
@click.option(
    "--prefix",
    "-p",
    required=False,
    type=str,
    help="Use prefix in filename.",
)
@click.option(
    "--keep",
    "-k",
    is_flag=True,
    default=False,
    type=bool,
    help="Do not delete files after backup.",
)
@click.option(
    "--auth",
    "-a",
    required=False,
    type=str,
    help="Bucket authentication.",
)
@click.option(
    "--bucket",
    "-b",
    required=False,
    type=str,
    help="Bucket name.",
)
def backup_and_push(dbname, no_filestore, **kwargs):
    options = {
        "filestore": not no_filestore,
        "ttype": kwargs.get("format"),
        "prefix": kwargs.get("prefix", False),
    }
    delete = True if not kwargs.get("keep", False) else True
    bucket = kwargs.get("bucket", False)
    auth = kwargs.get("auth", False)

    if auth:
        auth = eval(auth)

    res, filename, filepath = backup_database(
        dbname or settings.default_database, **options
    )
    if not res:
        sys.exit(1)

    upload_blob(
        bucket or settings.bucket_name,
        filepath,
        filename,
        auth or settings.json_auth,
    )

    if delete:
        clean_files(filepath)


cli.add_command(backup_and_push)
cli.add_command(backup)



from setuptools import find_packages, setup

setup(
    name="odoo-backup-tool",
    version="0.0.9",
    description="Odoo Backup Tool",
    long_description=open("README.md", encoding="utf-8").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/royaurelien/odoo-backup-tool",
    author="Aurelien ROY",
    author_email="roy.aurelien@gmail.com",
    license="BSD 2-clause",
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Build Tools",
        "License :: OSI Approved :: BSD License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
    ],
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        "click",
        "click-odoo-contrib",
        "google",
        "pydantic",
        "pydantic-settings",
    ],
    python_requires=">=3.8",
    entry_points={
        "console_scripts": [
            "obt = obt.cli.main:cli",
        ],
    },
)
